@@ -37,6 +37,7 @@ static GtkWidgetClass *parent_class;
 
 #define DEFAULT_RADIUS 64
 #define DEFAULT_BORDER 2
+#define DEFAULT_HAND_WIDTH 3
 
 /* Color usage */
 #define SELECTED_BG_COLOR(widget)	 (& (widget)->style->base[GTK_STATE_SELECTED])
@@ -54,6 +55,8 @@ struct _GpeClockFace
   guint radius;
   guint border;
   double hand_width;
+  gboolean radius_set;
+  gboolean hand_width_set;
 
   gboolean use_background_image;
   gboolean label_hours;
@@ -324,6 +327,13 @@ gpe_clock_face_expose (GtkWidget *widget,
 		 0,
 		 2 * M_PI);
       cairo_stroke (clock->cr);
+      cairo_arc (clock->cr,
+		 clock->x_offset + clock->radius,
+		 clock->y_offset + clock->radius,
+		 clock->hand_width,
+		 0,
+		 2 * M_PI);
+      cairo_fill (clock->cr);
 #else
       gdk_gc_set_foreground (gc, SELECTED_BG_COLOR ((GtkWidget*)clock));
       gdk_draw_arc (clock->backing_pixmap, gc, TRUE,
@@ -332,10 +342,10 @@ gpe_clock_face_expose (GtkWidget *widget,
 		    0, 360 * 64);
   
       gdk_draw_arc (clock->backing_pixmap, white_gc, TRUE,
-		    clock->x_offset + clock->border + clock->radius / 4,
-		    clock->y_offset + clock->border + clock->radius / 4,
-		    (3 * clock->radius / 4 - clock->border) * 2,
-		    (3 * clock->radius / 4 - clock->border) * 2,
+		    clock->x_offset + clock->border,
+		    clock->y_offset + clock->border,
+		    (clock->radius - clock->border) * 2,
+		    (clock->radius - clock->border) * 2,
 		    0, 360 * 64);
 #endif
 
@@ -590,10 +600,25 @@ gpe_clock_face_unprepare_xrender (GtkWidget *widget)
     }
 }
 
+static void
+gpe_clock_face_do_set_radius (GpeClockFace *clock, guint new_radius)
+{
+  clock->radius = new_radius;
+  gtk_widget_queue_resize (GTK_WIDGET (clock));
+	  
+  if (clock->radius != DEFAULT_RADIUS)
+    clock->use_background_image = FALSE;
+
+  if (!clock->hand_width_set)
+    clock->hand_width = new_radius / 40;
+}
+
 static void
 gpe_clock_face_size_allocate (GtkWidget     *widget,
 			      GtkAllocation *allocation)
 {
+  GpeClockFace *clock = GPE_CLOCK_FACE (widget);
+
   widget->allocation = *allocation;
 
   if (GTK_WIDGET_REALIZED (widget))
@@ -605,6 +630,18 @@ gpe_clock_face_size_allocate (GtkWidget     *widget,
       gpe_clock_face_unprepare_xrender (widget);
       gpe_clock_face_prepare_xrender (widget);
     }
+  
+  if (!clock->radius_set)
+    {
+      int min_size, new_radius;
+      
+      min_size = MIN (allocation->width, allocation->height);      
+
+      new_radius = (min_size / 2) - 1;
+
+      if (new_radius != clock->radius)
+	gpe_clock_face_do_set_radius (clock, new_radius);
+    }
 }
 
 static void
@@ -708,8 +745,8 @@ gpe_clock_face_size_request (GtkWidget	  *widget,
 {
   GpeClockFace *clock = GPE_CLOCK_FACE (widget);
 
-  requisition->height = (clock->radius + 3) * 2;
-  requisition->width = (clock->radius + 3) * 2;
+  requisition->height = (clock->radius + 1) * 2;
+  requisition->width = (clock->radius + 1) * 2;
 }
 
 static void
@@ -718,11 +755,13 @@ gpe_clock_face_init (GpeClockFace *clock)
   /* Double buffering doesn't play nicely with Render, so we will do that by hand */
   gtk_widget_set_double_buffered (GTK_WIDGET (clock), FALSE);
 
+  clock->radius_set = FALSE;
+  clock->hand_width_set = FALSE;
   clock->radius = DEFAULT_RADIUS;
   clock->border = DEFAULT_BORDER;
+  clock->hand_width = DEFAULT_HAND_WIDTH;
   clock->use_background_image = TRUE;
   clock->label_hours = FALSE;
-  clock->hand_width = 3;
 
   clock->backing_pixmap = NULL;
   clock->backing_gc = NULL;
@@ -760,9 +799,8 @@ gpe_clock_face_set_do_grabs (GpeClockFace *clock, gboolean yes)
 void
 gpe_clock_face_set_radius (GpeClockFace *clock, guint radius)
 {
-  clock->radius = radius;
-  if (clock->radius != DEFAULT_RADIUS)
-    clock->use_background_image = FALSE;
+  clock->radius_set = TRUE;
+  gpe_clock_face_do_set_radius (clock, radius);
 }
 
 void
@@ -781,6 +819,7 @@ void
 gpe_clock_face_set_hand_width (GpeClockFace *clock, double width)
 {
   clock->hand_width = width;
+  clock->hand_width_set = TRUE;
 }
 
 GType
